@@ -14,7 +14,6 @@
 #   limitations under the License.
 #
 
-
 import os
 import importlib.util
 
@@ -22,7 +21,7 @@ from requests.exceptions import ConnectionError
 
 from hsfs.decorators import connected, not_connected
 from hsfs import engine, client
-from hsfs.core import feature_store_api, project_api, hosts_api, services_api
+from hsfs.core import feature_store_api, project_api, hosts_api, services_api, rules_api
 
 AWS_DEFAULT_REGION = "default"
 HOPSWORKS_PORT_DEFAULT = 443
@@ -49,6 +48,27 @@ class Connection:
         conn = hsfs.connection()
         ```
 
+    !!! hint "Save API Key as File"
+        To get started quickly, without saving the Hopsworks API in a secret storage,
+        you can simply create a file with the previously created Hopsworks API Key and
+        place it on the environment from which you wish to connect to the Hopsworks
+        Feature Store.
+
+        You can then connect by simply passing the path to the key file when
+        instantiating a connection:
+
+        ```python hl_lines="6"
+            import hsfs
+            conn = hsfs.connection(
+                'my_instance',                      # DNS of your Feature Store instance
+                443,                                # Port to reach your Hopsworks instance, defaults to 443
+                'my_project',                       # Name of your Hopsworks Feature Store project
+                api_key_file='featurestore.key',    # The file containing the API key generated above
+                hostname_verification=True)         # Disable for self-signed certificates
+            )
+            fs = conn.get_feature_store()           # Get the project's default feature store
+        ```
+
     Clients in external clusters need to connect to the Hopsworks Feature Store using an
     API key. The API key is generated inside the Hopsworks platform, and requires at
     least the "project" and "featurestore" scopes to be able to access a feature store.
@@ -115,6 +135,7 @@ class Connection:
         self._api_key_file = api_key_file
         self._api_key_value = api_key_value
         self._connected = False
+        self._rules_api = rules_api.RulesApi()
 
         self.connect()
 
@@ -239,7 +260,7 @@ class Connection:
         Databricks cluster to a Hopsworks Feature Store instance. It returns a
         `Connection` object and will print instructions on how to finalize the setup
         of the Databricks cluster.
-        See also the [Databricks integration guide](../integrations/databricks.md).
+        See also the [Databricks integration guide](../integrations/databricks/configuration.md).
         """
         connection = cls(
             host,
@@ -465,6 +486,16 @@ class Connection:
     def api_key_value(self, api_key_value):
         self._api_key_value = api_key_value
 
+    def get_rules(self):
+        """Get a rule with a certain name or all rules available for data validation."""
+
+        return self._rules_api.get()
+
+    def get_rule(self, name: str):
+        """Get a rule with a certain name or all rules available for data validation."""
+
+        return self._rules_api.get(name)
+
     def __enter__(self):
         self.connect()
         return self
