@@ -13,16 +13,15 @@
 #   limitations under the License.
 #
 
-from hsfs import engine
+import warnings
+
+from hsfs import engine, client, util
 from hsfs import feature_group as fg
-from hsfs.core import feature_group_base_engine, hudi_engine
 from hsfs.client import exceptions
+from hsfs.core import feature_group_base_engine, hudi_engine
 
 
 class FeatureGroupEngine(feature_group_base_engine.FeatureGroupBaseEngine):
-    OVERWRITE = "overwrite"
-    APPEND = "append"
-
     def save(self, feature_group, feature_dataframe, write_options):
 
         if len(feature_group.features) == 0:
@@ -38,29 +37,24 @@ class FeatureGroupEngine(feature_group_base_engine.FeatureGroupBaseEngine):
                 feat.primary = True
             if feat.name in feature_group.partition_key:
                 feat.partition = True
+            if (
+                feature_group.hudi_precombine_key is not None
+                and feat.name == feature_group.hudi_precombine_key
+            ):
+                feat.hudi_precombine_key = True
 
         self._feature_group_api.save(feature_group)
+        validation_id = None
+        if feature_group.validation_type != "NONE":
+            validation = feature_group.validate(feature_dataframe)
+            validation_id = validation.validation_id
 
         offline_write_options = write_options
-        online_write_options = write_options
-
-        table_name = self._get_table_name(feature_group)
-
-        if feature_group.online_enabled:
-            # Add JDBC connection configuration in case of online feature group
-            online_conn = self._storage_connector_api.get_online_connector()
-
-            jdbc_options = online_conn.spark_options()
-            jdbc_options["dbtable"] = self._get_online_table_name(feature_group)
-            jdbc_options["driver"] = "com.mysql.cj.jdbc.Driver"
-
-            online_write_options = {**jdbc_options, **online_write_options}
+        online_write_options = self.get_kafka_config(write_options)
 
         engine.get_instance().save_dataframe(
-            table_name,
             feature_group,
             feature_dataframe,
-            self.APPEND,
             hudi_engine.HudiEngine.HUDI_BULK_INSERT
             if feature_group.time_travel_format == "HUDI"
             else None,
@@ -68,6 +62,7 @@ class FeatureGroupEngine(feature_group_base_engine.FeatureGroupBaseEngine):
             None,
             offline_write_options,
             online_write_options,
+            validation_id,
         )
 
     def insert(
@@ -79,58 +74,57 @@ class FeatureGroupEngine(feature_group_base_engine.FeatureGroupBaseEngine):
         storage,
         write_options,
     ):
+        validation_id = None
+        if feature_group.validation_type != "NONE":
+            validation = feature_group.validate(feature_dataframe)
+            validation_id = validation.validation_id
+
         offline_write_options = write_options
-        online_write_options = write_options
+        online_write_options = self.get_kafka_config(write_options)
 
         if not feature_group.online_enabled and storage == "online":
             raise exceptions.FeatureStoreException(
                 "Online storage is not enabled for this feature group."
             )
-        elif (
-            feature_group.online_enabled and storage != "offline"
-        ) or storage == "online":
-            # Add JDBC connection configuration in case of online feature group
-            online_conn = self._storage_connector_api.get_online_connector()
-
-            jdbc_options = online_conn.spark_options()
-            jdbc_options["dbtable"] = self._get_online_table_name(feature_group)
-            jdbc_options["driver"] = "com.mysql.cj.jdbc.Driver"
-
-            online_write_options = {**jdbc_options, **online_write_options}
 
         if overwrite:
             self._feature_group_api.delete_content(feature_group)
 
         engine.get_instance().save_dataframe(
-            self._get_table_name(feature_group),
             feature_group,
             feature_dataframe,
-            self.APPEND,
             "bulk_insert" if overwrite else operation,
             feature_group.online_enabled,
             storage,
             offline_write_options,
             online_write_options,
+            validation_id,
         )
 
     def delete(self, feature_group):
         self._feature_group_api.delete(feature_group)
 
-    def commit_details(self, feature_group, limit):
-        hudi_engine_instance = hudi_engine.HudiEngine(
-            feature_group.feature_store_id,
-            feature_group.feature_store_name,
-            feature_group,
-            engine.get_instance()._spark_context,
-            engine.get_instance()._spark_session,
+    def commit_details(self, feature_group, wallclock_time, limit):
+        if (
+            feature_group._time_travel_format is None
+            or feature_group._time_travel_format.upper() != "HUDI"
+        ):
+            raise exceptions.FeatureStoreException(
+                "commit_details can only be used on time travel enabled feature groups"
+            )
+
+        wallclock_timestamp = (
+            util.get_timestamp_from_date_string(wallclock_time)
+            if wallclock_time is not None
+            else None
         )
-        feature_group_commits = self._feature_group_api.commit_details(
-            feature_group, limit
+        feature_group_commits = self._feature_group_api.get_commit_details(
+            feature_group, wallclock_timestamp, limit
         )
         commit_details = {}
         for feature_group_commit in feature_group_commits:
             commit_details[feature_group_commit.commitid] = {
-                "committedOn": hudi_engine_instance._timestamp_to_hudiformat(
+                "committedOn": util.get_hudi_datestr_from_timestamp(
                     feature_group_commit.commitid
                 ),
                 "rowsUpdated": feature_group_commit.rows_updated,
@@ -150,24 +144,15 @@ class FeatureGroupEngine(feature_group_base_engine.FeatureGroupBaseEngine):
         )
         return hudi_engine_instance.delete_record(delete_df, write_options)
 
-    def update_statistics_config(self, feature_group):
-        """Update the statistics configuration of a feature group."""
+    def update_validation_type(self, feature_group):
+        """Update the validation type specified of the feature group ."""
         self._feature_group_api.update_metadata(
-            feature_group, feature_group, "updateStatsSettings"
+            feature_group,
+            feature_group,
+            "validationType",
+            feature_group.validation_type,
         )
 
-    def _get_table_name(self, feature_group):
-        return (
-            feature_group.feature_store_name
-            + "."
-            + feature_group.name
-            + "_"
-            + str(feature_group.version)
-        )
-
-    def _get_online_table_name(self, feature_group):
-        return feature_group.name + "_" + str(feature_group.version)
-
     def sql(self, query, feature_store_name, dataframe_type, online):
         if online:
             online_conn = self._storage_connector_api.get_online_connector()
@@ -182,10 +167,10 @@ class FeatureGroupEngine(feature_group_base_engine.FeatureGroupBaseEngine):
         # perform changes on copy in case the update fails, so we don't leave
         # the user object in corrupted state
         copy_feature_group = fg.FeatureGroup(
-            None,
-            None,
-            None,
-            None,
+            name=None,
+            version=None,
+            featurestore_id=None,
+            description=None,
             id=feature_group.id,
             features=feature_group.features + new_features,
         )
@@ -196,13 +181,68 @@ class FeatureGroupEngine(feature_group_base_engine.FeatureGroupBaseEngine):
     def update_description(self, feature_group, description):
         """Updates the description of a feature group."""
         copy_feature_group = fg.FeatureGroup(
-            None,
-            None,
-            description,
-            None,
+            name=None,
+            version=None,
+            featurestore_id=None,
+            description=description,
             id=feature_group.id,
             features=feature_group.features,
         )
         self._feature_group_api.update_metadata(
             feature_group, copy_feature_group, "updateMetadata"
         )
+
+    def get_avro_schema(self, feature_group):
+        return self._kafka_api.get_topic_subject(feature_group._online_topic_name)
+
+    def get_kafka_config(self, online_write_options):
+        config = {
+            "kafka.bootstrap.servers": ",".join(
+                [
+                    endpoint.replace("INTERNAL://", "")
+                    for endpoint in self._kafka_api.get_broker_endpoints()
+                ]
+            ),
+            "kafka.security.protocol": "SSL",
+            "kafka.ssl.truststore.location": client.get_instance()._get_jks_trust_store_path(),
+            "kafka.ssl.truststore.password": client.get_instance()._cert_key,
+            "kafka.ssl.keystore.location": client.get_instance()._get_jks_key_store_path(),
+            "kafka.ssl.keystore.password": client.get_instance()._cert_key,
+            "kafka.ssl.key.password": client.get_instance()._cert_key,
+            "kafka.ssl.endpoint.identification.algorithm": "",
+        }
+        return {**online_write_options, **config}
+
+    def insert_stream(
+        self,
+        feature_group,
+        dataframe,
+        query_name,
+        output_mode,
+        await_termination,
+        timeout,
+        write_options,
+    ):
+        if not feature_group.online_enabled:
+            raise exceptions.FeatureStoreException(
+                "Online storage is not enabled for this feature group. "
+                "It is currently only possible to stream to the online storage."
+            )
+
+        if feature_group.validation_type != "NONE":
+            warnings.warn(
+                "Stream ingestion for feature group `{}`, with version `{}` will not perform validation.".format(
+                    feature_group.name, feature_group.version
+                ),
+                util.ValidationWarning,
+            )
+
+        return engine.get_instance().save_stream_dataframe(
+            feature_group,
+            dataframe,
+            query_name,
+            output_mode,
+            await_termination,
+            timeout,
+            self.get_kafka_config(write_options),
+        )
