@@ -22,14 +22,16 @@ from hsfs import (
     training_dataset,
     feature_group,
     feature,
-    on_demand_feature_group,
     util,
     storage_connector,
+    expectation,
+    rule,
 )
 from hsfs.core import (
     feature_group_api,
     storage_connector_api,
     training_dataset_api,
+    expectations_api,
     feature_group_engine,
 )
 from hsfs.statistics_config import StatisticsConfig
@@ -51,6 +53,9 @@ class FeatureStore:
         offline_featurestore_name,
         hive_endpoint,
         online_enabled,
+        num_feature_groups=None,
+        num_training_datasets=None,
+        num_storage_connectors=None,
         online_featurestore_name=None,
         mysql_server_endpoint=None,
         online_featurestore_size=None,
@@ -69,12 +74,16 @@ class FeatureStore:
         self._hive_endpoint = hive_endpoint
         self._mysql_server_endpoint = mysql_server_endpoint
         self._online_enabled = online_enabled
+        self._num_feature_groups = num_feature_groups
+        self._num_training_datasets = num_training_datasets
+        self._num_storage_connectors = num_storage_connectors
 
         self._feature_group_api = feature_group_api.FeatureGroupApi(self._id)
         self._storage_connector_api = storage_connector_api.StorageConnectorApi(
             self._id
         )
         self._training_dataset_api = training_dataset_api.TrainingDatasetApi(self._id)
+        self._expectations_api = expectations_api.ExpectationsApi(self._id)
 
         self._feature_group_engine = feature_group_engine.FeatureGroupEngine(self._id)
 
@@ -117,13 +126,13 @@ class FeatureStore:
     def get_on_demand_feature_group(self, name: str, version: int = None):
         """Get a on-demand feature group entity from the feature store.
 
-        Getting a on-demand feature group from the Feature Store means getting its 
-        metadata handle so you can subsequently read the data into a Spark or 
+        Getting a on-demand feature group from the Feature Store means getting its
+        metadata handle so you can subsequently read the data into a Spark or
         Pandas DataFrame or use the `Query`-API to perform joins between feature groups.
 
         # Arguments
             name: Name of the on-demand feature group to get.
-            version: Version of the on-demand feature group to retrieve, 
+            version: Version of the on-demand feature group to retrieve,
                 defaults to `None` and will return the `version=1`.
 
         # Returns
@@ -174,7 +183,7 @@ class FeatureStore:
             version = self.DEFAULT_VERSION
         return self._training_dataset_api.get(name, version)
 
-    def get_storage_connector(self, name: str, connector_type: str):
+    def get_storage_connector(self, name: str):
         """Get a previously created storage connector from the feature store.
 
         Storage connectors encapsulate all information needed for the execution engine
@@ -188,20 +197,18 @@ class FeatureStore:
         !!! example "Getting a Storage Connector"
             ```python
 
-            sc = fs.get_storage_connector("demo_fs_meb10000_Training_Datasets", "HOPSFS")
+            sc = fs.get_storage_connector("demo_fs_meb10000_Training_Datasets")
 
             td = fs.create_training_dataset(..., storage_connector=sc, ...)
             ```
 
         # Arguments
             name: Name of the storage connector to retrieve.
-            connector_type: Type of the storage connector, e.g. `"JDBC"`, `"HOPSFS"`
-                or `"S3"`.
 
         # Returns
             `StorageConnector`. Storage connector object.
         """
-        return self._storage_connector_api.get(name, connector_type)
+        return self._storage_connector_api.get(name)
 
     def sql(self, query, dataframe_type="default", online=False):
         return self._feature_group_engine.sql(query, self._name, dataframe_type, online)
@@ -217,6 +224,39 @@ class FeatureStore:
         """
         return self._storage_connector_api.get_online_connector()
 
+    def get_expectation(self, name: str):
+        """Get an expectation entity from the feature store.
+
+        Getting an expectation from the Feature Store means getting its metadata handle
+        so you can subsequently add features and/or rules and save it which will overwrite the previous instance.
+
+        # Arguments
+            name: Name of the training dataset to get.
+
+        # Returns
+            `Expectation`: The expectation metadata object.
+
+        # Raises
+            `RestAPIError`: If unable to retrieve the expectation from the feature store.
+        """
+
+        return self._expectations_api.get(name)
+
+    def get_expectations(self):
+        """Get all expectation entities from the feature store.
+
+        Getting expectations from the Feature Store means getting their metadata handles
+        so you can subsequently add features and/or rules and save it which will overwrite the previous instance.
+
+        # Returns
+            `Expectation`: The expectation metadata object.
+
+        # Raises
+            `RestAPIError`: If unable to retrieve the expectations from the feature store.
+        """
+
+        return self._expectations_api.get()
+
     def create_feature_group(
         self,
         name: str,
@@ -226,8 +266,11 @@ class FeatureStore:
         time_travel_format: Optional[str] = "HUDI",
         partition_key: Optional[List[str]] = [],
         primary_key: Optional[List[str]] = [],
+        hudi_precombine_key: Optional[str] = None,
         features: Optional[List[feature.Feature]] = [],
         statistics_config: Optional[Union[StatisticsConfig, bool, dict]] = None,
+        validation_type: Optional[str] = "NONE",
+        expectations: Optional[List[expectation.Expectation]] = [],
     ):
         """Create a feature group metadata object.
 
@@ -257,6 +300,10 @@ class FeatureStore:
                 features and will be used as joining key, if not specified otherwise.
                 Defaults to empty list `[]`, and the first column of the DataFrame will
                 be used as primary key.
+            hudi_precombine_key: A feature name to be used as a precombine key for the `"HUDI"`
+                feature group. Defaults to `None`. If feature group has time travel format
+                `"HUDI"` and hudi precombine key was not specified then the first primary key of
+                the feature group will be used as hudi precombine key.
             features: Optionally, define the schema of the feature group manually as a
                 list of `Feature` objects. Defaults to empty list `[]` and will use the
                 schema information of the DataFrame provided in the `save` method.
@@ -267,6 +314,12 @@ class FeatureStore:
                 values should be booleans indicating the setting. To fully turn off
                 statistics computation pass `statistics_config=False`. Defaults to
                 `None` and will compute only descriptive statistics.
+            validation_type: Optionally, set the validation type to one of "NONE", "STRICT",
+                "WARNING", "ALL". Determines the mode in which data validation is applied on
+                 ingested or already existing feature group data.
+            expectations: Optionally, a list of expectations to be attached to the feature group.
+                The expectations list contains Expectation metadata objects which can be retrieved with
+                the `get_expectation()` and `get_expectations()` functions.
 
         # Returns
             `FeatureGroup`. The feature group metadata object.
@@ -279,20 +332,27 @@ class FeatureStore:
             time_travel_format=time_travel_format,
             partition_key=partition_key,
             primary_key=primary_key,
+            hudi_precombine_key=hudi_precombine_key,
             featurestore_id=self._id,
             featurestore_name=self._name,
             features=features,
             statistics_config=statistics_config,
+            validation_type=validation_type,
+            expectations=expectations,
         )
 
     def create_on_demand_feature_group(
         self,
         name: str,
-        query: str,
         storage_connector: storage_connector.StorageConnector,
+        query: Optional[str] = None,
+        data_format: Optional[str] = None,
+        path: Optional[str] = "",
+        options: Optional[Dict[str, str]] = {},
         version: Optional[int] = None,
         description: Optional[str] = "",
         features: Optional[List[feature.Feature]] = [],
+        statistics_config: Optional[Union[StatisticsConfig, bool, dict]] = None,
     ):
         """Create a on-demand feature group metadata object.
 
@@ -304,9 +364,13 @@ class FeatureStore:
         # Arguments
             name: Name of the on-demand feature group to create.
             query: A string containing a SQL query valid for the target data source.
-                the query will be used to pull data from the data sources when the 
+                the query will be used to pull data from the data sources when the
                 feature group is used.
-            storage_connector: the storage connector to use to establish connectivity 
+            data_format: If the on-demand feature groups refers to a directory with data,
+                the data format to use when reading it
+            path: The location within the scope of the storage connector, from where to read
+                the data for the on-demand feature group
+            storage_connector: the storage connector to use to establish connectivity
                 with the data source.
             version: Version of the on-demand feature group to retrieve, defaults to `None` and
                 will create the feature group with incremented version from the last
@@ -318,16 +382,30 @@ class FeatureStore:
                 list of `Feature` objects. Defaults to empty list `[]` and will use the
                 schema information of the DataFrame resulting by executing the provided query
                 against the data source.
+            statistics_config: A configuration object, or a dictionary with keys
+                "`enabled`" to generally enable descriptive statistics computation for
+                this on-demand feature group, `"correlations`" to turn on feature correlation
+                computation and `"histograms"` to compute feature value frequencies. The
+                values should be booleans indicating the setting. To fully turn off
+                statistics computation pass `statistics_config=False`. Defaults to
+                `None` and will compute only descriptive statistics.
+
+        # Returns
+            `OnDemandFeatureGroup`. The on-demand feature group metadata object.
         """
-        return on_demand_feature_group.OnDemandFeatureGroup(
+        return feature_group.OnDemandFeatureGroup(
             name=name,
             query=query,
+            data_format=data_format,
+            path=path,
+            options=options,
             storage_connector=storage_connector,
             version=version,
             description=description,
             featurestore_id=self._id,
             featurestore_name=self._name,
             features=features,
+            statistics_config=statistics_config,
         )
 
     def create_training_dataset(
@@ -336,6 +414,7 @@ class FeatureStore:
         version: Optional[int] = None,
         description: Optional[str] = "",
         data_format: Optional[str] = "tfrecords",
+        coalesce: Optional[bool] = False,
         storage_connector: Optional[storage_connector.StorageConnector] = None,
         splits: Optional[Dict[str, float]] = {},
         location: Optional[str] = "",
@@ -375,6 +454,9 @@ class FeatureStore:
                 `""`.
             data_format: The data format used to save the training dataset,
                 defaults to `"tfrecords"`-format.
+            coalesce: If true the training dataset data will be coalesced into
+                a single partition before writing. The resulting training dataset
+                will be a single file per split. Default False.
             storage_connector: Storage connector defining the sink location for the
                 training dataset, defaults to `None`, and materializes training dataset
                 on HopsFS.
@@ -417,4 +499,87 @@ class FeatureStore:
             seed=seed,
             statistics_config=statistics_config,
             label=label,
+            coalesce=coalesce,
         )
+
+    def create_expectation(
+        self,
+        name: str,
+        description: Optional[str] = "",
+        features: Optional[List[str]] = [],
+        rules: Optional[List[rule.Rule]] = [],
+    ):
+        """Create an expectation metadata object.
+
+        !!! note "Lazy"
+            This method is lazy and does not persist the expectation in the
+            feature store on its own. To materialize the expectation and save
+            call the `save()` method of the expectation metadata object.
+
+        # Arguments
+            name: Name of the training dataset to create.
+            description: A string describing the expectation that can describe its business logic and applications
+                within the feature store.
+            features: The features this expectation is applied on.
+            rules: The validation rules this expectation will apply to the its features.
+
+        # Returns:
+            `Expectation`: The expectation metadata object.
+        """
+        return expectation.Expectation(
+            name=name,
+            description=description,
+            features=features,
+            rules=rules,
+            featurestore_id=self._id,
+        )
+
+    @property
+    def id(self):
+        """Id of the feature store."""
+        return self._id
+
+    @property
+    def name(self):
+        """Name of the feature store."""
+        return self._name
+
+    @property
+    def project_name(self):
+        """Name of the project in which the feature store is located."""
+        return self._project_name
+
+    @property
+    def project_id(self):
+        """Id of the project in which the feature store is located."""
+        return self._project_id
+
+    @property
+    def description(self):
+        """Description of the feature store."""
+        return self._description
+
+    @property
+    def online_featurestore_name(self):
+        """Name of the online feature store database."""
+        return self._online_feature_store_name
+
+    @property
+    def mysql_server_endpoint(self):
+        """MySQL server endpoint for the online feature store."""
+        return self._mysql_server_endpoint
+
+    @property
+    def online_enabled(self):
+        """Indicator whether online feature store is enabled."""
+        return self._online_enabled
+
+    @property
+    def hive_endpoint(self):
+        """Hive endpoint for the offline feature store."""
+        return self._hive_endpoint
+
+    @property
+    def offline_featurestore_name(self):
+        """Name of the offline feature store database."""
+        return self._offline_feature_store_name
