@@ -26,14 +26,12 @@ from hsfs import util, engine, training_dataset_feature
 from hsfs.statistics_config import StatisticsConfig
 from hsfs.storage_connector import StorageConnector
 from hsfs.core import (
-    query,
     training_dataset_api,
-    storage_connector_api,
     training_dataset_engine,
     tfdata_engine,
     statistics_engine,
 )
-from hsfs.client import exceptions
+from hsfs.constructor import query
 
 
 class TrainingDataset:
@@ -48,6 +46,7 @@ class TrainingDataset:
         data_format,
         location,
         featurestore_id,
+        coalesce=False,
         description=None,
         storage_connector=None,
         splits=None,
@@ -58,11 +57,7 @@ class TrainingDataset:
         statistics_config=None,
         featurestore_name=None,
         id=None,
-        jobs=None,
         inode_id=None,
-        storage_connector_name=None,
-        storage_connector_id=None,
-        storage_connector_type=None,
         training_dataset_type=None,
         from_query=None,
         querydto=None,
@@ -73,10 +68,15 @@ class TrainingDataset:
         self._version = version
         self._description = description
         self._data_format = data_format
+        self._coalesce = coalesce
         self._seed = seed
         self._location = location
         self._from_query = from_query
         self._querydto = querydto
+        self._feature_store_id = featurestore_id
+        self._prepared_statement_connection = None
+        self._prepared_statements = None
+        self._serving_keys = None
 
         self._training_dataset_api = training_dataset_api.TrainingDatasetApi(
             featurestore_id
@@ -86,10 +86,6 @@ class TrainingDataset:
             featurestore_id
         )
 
-        self._storage_connector_api = storage_connector_api.StorageConnectorApi(
-            featurestore_id
-        )
-
         self._statistics_engine = statistics_engine.StatisticsEngine(
             featurestore_id, self.ENTITY_TYPE
         )
@@ -105,17 +101,20 @@ class TrainingDataset:
         else:
             # type available -> init from backend response
             # make rest call to get all connector information, description etc.
-            self._storage_connector = self._storage_connector_api.get_by_id(
-                storage_connector_id, storage_connector_type
+            self._storage_connector = StorageConnector.from_response_json(
+                storage_connector
             )
+
             self._features = [
                 training_dataset_feature.TrainingDatasetFeature.from_response_json(feat)
                 for feat in features
             ]
             self._splits = splits
             self._training_dataset_type = training_dataset_type
-            self.statistics_config = None
-            self._label = [feat.name for feat in self._features if feat.label]
+            self._statistics_config = StatisticsConfig.from_response_json(
+                statistics_config
+            )
+            self._label = [feat.name.lower() for feat in self._features if feat.label]
 
     def save(
         self,
@@ -148,27 +147,13 @@ class TrainingDataset:
         # Raises
             `RestAPIError`: Unable to create training dataset metadata.
         """
-        if isinstance(features, query.Query):
-            feature_dataframe = features.read()
-            self._querydto = features
-        else:
-            feature_dataframe = engine.get_instance().convert_to_default_dataframe(
-                features
-            )
-
-        self._features = engine.get_instance().parse_schema_training_dataset(
-            feature_dataframe
-        )
-
-        self._set_label_features()
-
         user_version = self._version
         user_stats_config = self._statistics_config
-        self._training_dataset_engine.save(self, feature_dataframe, write_options)
+        self._training_dataset_engine.save(self, features, write_options)
         # currently we do not save the training dataset statistics config for training datasets
         self.statistics_config = user_stats_config
-        if self.statistics_config.enabled:
-            self._statistics_engine.compute_statistics(self, feature_dataframe)
+        if self.statistics_config.enabled and engine.get_type() == "spark":
+            self._statistics_engine.compute_statistics(self, self.read())
         if user_version is None:
             warnings.warn(
                 "No version provided for creating training dataset `{}`, incremented version to `{}`.".format(
@@ -213,15 +198,7 @@ class TrainingDataset:
         # Raises
             `RestAPIError`: Unable to create training dataset metadata.
         """
-        if isinstance(features, query.Query):
-            feature_dataframe = features.read()
-        else:
-            feature_dataframe = engine.get_instance().convert_to_default_dataframe(
-                features
-            )
-        self._training_dataset_engine.insert(
-            self, feature_dataframe, write_options, overwrite
-        )
+        self._training_dataset_engine.insert(self, features, write_options, overwrite)
 
         self.compute_statistics()
 
@@ -244,7 +221,7 @@ class TrainingDataset:
         """Recompute the statistics for the training dataset and save them to the
         feature store.
         """
-        if self.statistics_config.enabled:
+        if self.statistics_config.enabled and engine.get_type() == "spark":
             return self._statistics_engine.compute_statistics(self, self.read())
 
     def tf_data(
@@ -293,40 +270,89 @@ class TrainingDataset:
         """
         self.read(split).show(n)
 
-    def add_tag(self, name: str, value: str = None):
-        """Attach a name/value tag to a training dataset.
+    def add_tag(self, name: str, value):
+        """Attach a tag to a training dataset.
 
-        A tag can consist of a name only or a name/value pair. Tag names are
-        unique identifiers.
+        A tag consists of a <name,value> pair. Tag names are unique identifiers across the whole cluster.
+        The value of a tag can be any valid json - primitives, arrays or json objects.
 
         # Arguments
             name: Name of the tag to be added.
-            value: Value of the tag to be added, defaults to `None`.
+            value: Value of the tag to be added.
+
+        # Raises
+            `RestAPIError` in case the backend fails to add the tag.
         """
         self._training_dataset_engine.add_tag(self, name, value)
 
     def delete_tag(self, name: str):
-        """Delete a tag from a training dataset.
-
-        Tag names are unique identifiers.
+        """Delete a tag attached to a training dataset.
 
         # Arguments
             name: Name of the tag to be removed.
+
+        # Raises
+            `RestAPIError` in case the backend fails to delete the tag.
         """
         self._training_dataset_engine.delete_tag(self, name)
 
-    def get_tag(self, name=None):
+    def get_tag(self, name):
         """Get the tags of a training dataset.
 
-        Tag names are unique identifiers. Returns all tags if no tag name is
-        specified.
-
         # Arguments
-            name: Name of the tag to get, defaults to `None`.
+            name: Name of the tag to get.
+
         # Returns
-            `List[Tag]`. List of tags as name/value pairs.
+            tag value
+
+        # Raises
+            `RestAPIError` in case the backend fails to retrieve the tag.
         """
-        return self._training_dataset_engine.get_tags(self, name)
+        return self._training_dataset_engine.get_tag(self, name)
+
+    def get_tags(self):
+        """Returns all tags attached to a training dataset.
+
+        # Returns
+            `Dict[str, obj]` of tags.
+
+        # Raises
+            `RestAPIError` in case the backend fails to retrieve the tags.
+        """
+        return self._training_dataset_engine.get_tags(self)
+
+    def update_statistics_config(self):
+        """Update the statistics configuration of the training dataset.
+
+        Change the `statistics_config` object and persist the changes by calling
+        this method.
+
+        # Returns
+            `TrainingDataset`. The updated metadata object of the training dataset.
+
+        # Raises
+            `RestAPIError`.
+        """
+        self._training_dataset_engine.update_statistics_config(self)
+        return self
+
+    def delete(self):
+        """Delete training dataset and all associated metadata.
+
+        !!! note "Drops only HopsFS data"
+            Note that this operation drops only files which were materialized in
+            HopsFS. If you used a Storage Connector for a cloud storage such as S3,
+            the data will not be deleted, but you will not be able to track it anymore
+            from the Feature Store.
+
+        !!! danger "Potentially dangerous operation"
+            This operation drops all metadata associated with **this version** of the
+            training dataset **and** and the materialized data in HopsFS.
+
+        # Raises
+            `RestAPIError`.
+        """
+        self._training_dataset_api.delete(self)
 
     @classmethod
     def from_response_json(cls, json_dict):
@@ -342,12 +368,13 @@ class TrainingDataset:
         return self
 
     def _infer_training_dataset_type(self, connector_type):
-        if connector_type == StorageConnector.HOPSFS:
+        if connector_type == StorageConnector.HOPSFS or connector_type is None:
             return self.HOPSFS
-        elif connector_type == StorageConnector.S3:
+        elif (
+            connector_type == StorageConnector.S3
+            or connector_type == StorageConnector.ADLS
+        ):
             return self.EXTERNAL
-        elif connector_type is None:
-            return self.HOPSFS
         else:
             raise TypeError(
                 "Storage connectors of type {} are currently not supported for training datasets.".format(
@@ -364,13 +391,15 @@ class TrainingDataset:
             "version": self._version,
             "description": self._description,
             "dataFormat": self._data_format,
-            "storageConnectorId": self._storage_connector.id,
+            "coalesce": self._coalesce,
+            "storageConnector": self._storage_connector,
             "location": self._location,
             "trainingDatasetType": self._training_dataset_type,
             "features": self._features,
             "splits": self._splits,
             "seed": self._seed,
             "queryDTO": self._querydto.to_dict() if self._querydto else None,
+            "statisticsConfig": self._statistics_config,
         }
 
     @property
@@ -418,6 +447,17 @@ class TrainingDataset:
     def data_format(self, data_format):
         self._data_format = data_format
 
+    @property
+    def coalesce(self):
+        """If true the training dataset data will be coalesced into
+        a single partition before writing. The resulting training dataset
+        will be a single file per split"""
+        return self._coalesce
+
+    @coalesce.setter
+    def coalesce(self, coalesce):
+        self._coalesce = coalesce
+
     @property
     def write_options(self):
         """User provided options to write training dataset."""
@@ -550,6 +590,23 @@ class TrainingDataset:
         """
         return self._training_dataset_engine.query(self, online, with_label)
 
+    def init_prepared_statement(self):
+        """Initialise and cache parametrised prepared statement to retrieve feature vector from online feature store."""
+        if self.prepared_statements is None:
+            self._training_dataset_engine.init_prepared_statement(self)
+
+    def get_serving_vector(self, entry: Dict[str, Any]):
+        """Returns assembled serving vector from online feature store.
+
+        # Arguments
+            entry: dictionary of training dataset feature group primary key names as keys and values provided by
+            serving application.
+        # Returns
+            `list` List of feature values related to provided primary keys, ordered according to positions of this
+            features in training dataset query.
+        """
+        return self._training_dataset_engine.get_serving_vector(self, entry)
+
     @property
     def label(self):
         """The label/prediction feature of the training dataset.
@@ -560,19 +617,37 @@ class TrainingDataset:
 
     @label.setter
     def label(self, label):
-        self._label = label
+        self._label = [lb.lower() for lb in label]
 
-    def _set_label_features(self):
-        for f_name in self._label:
-            found = False
-            for f in self._features:
-                if f_name == f.name:
-                    f.label = True
-                    found = True
-                    break
-            if not found:
-                raise exceptions.FeatureStoreException(
-                    "The specified label `{}` could not be found among the features: {}.".format(
-                        f_name, [feat.name for feat in self._features]
-                    )
-                )
+    @property
+    def feature_store_id(self):
+        return self._feature_store_id
+
+    @property
+    def prepared_statement_connection(self):
+        """JDBC connection to online features store."""
+        return self._prepared_statement_connection
+
+    @prepared_statement_connection.setter
+    def prepared_statement_connection(self, prepared_statement_connection):
+        self._prepared_statement_connection = prepared_statement_connection
+
+    @property
+    def prepared_statements(self):
+        """The dict object of prepared_statements as values and kes as indices of positions in the query for
+        selecting features from feature groups of the training dataset.
+        """
+        return self._prepared_statements
+
+    @prepared_statements.setter
+    def prepared_statements(self, prepared_statements):
+        self._prepared_statements = prepared_statements
+
+    @property
+    def serving_keys(self):
+        """Set of primary key names that is used as keys in input dict object for `get_serving_vector` method."""
+        return self._serving_keys
+
+    @serving_keys.setter
+    def serving_keys(self, serving_vector_keys):
+        self._serving_keys = serving_vector_keys
